package com.vendor.burn.sparse;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.log.Log;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.serial.workers.CallLibBootromWorker;
import com.vendor.burn.sparse.SparseChunkHeader;
import com.vendor.burn.sparse.SparseHeader;
import com.vendor.burn.utilitily.BytesUtils;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;

public class SparseFileAgent {
  static final int FILE_BLOCK_SIZE_32 = 33554432;
  
  static final int SEPARATE_FILE_XML_LIST = 30000;
  
  static final int SEPARATE_CHUNKLEN = 5242880;
  
  public static List<PartitionItem> getSparseFileXmlList(long ddrUsableSize, PartitionItem partitionItem, boolean isTftpOccupied) throws IOException {
    if (ddrUsableSize > 33554432L)
      CallLibBootromWorker.setPacketSize((int)ddrUsableSize); 
    String fileName = partitionItem.getFile().getCanonicalPath();
    File partitionItemFile = partitionItem.getFile();
    boolean isDeleteTempFile = false;
    String canSplitFilePath = getCanSplitFilePath(isTftpOccupied, fileName, partitionItemFile);
    if (canSplitFilePath.isEmpty())
      return new ArrayList<>(); 
    if (!fileName.equals(canSplitFilePath))
      isDeleteTempFile = true; 
    fileName = canSplitFilePath;
    byte[] sparseFileXmlList = new byte[30000];
    try {
      if (!CallLibBootromWorker.splitSparseFile(fileName, partitionItem.getFileSystem().name(), partitionItem
          .getStart(), partitionItem.getLength())) {
        Log.error(NLS.bind(Messages.fileSpiltFailed, fileName));
        return new ArrayList();
      } 
    } finally {
      CallLibBootromWorker.getSparseFileXmlList(fileName, sparseFileXmlList);
      if (isDeleteTempFile)
        FileUtils.deleteFile(new File(canSplitFilePath)); 
    } 
    return getPartitionItemList(fileName, sparseFileXmlList);
  }
  
  private static String getCanSplitFilePath(boolean isTftpOccupied, String fileName, File partitionItemFile) {
    if (fileName.startsWith("\\\\") && partitionItemFile.exists()) {
      if (canWriteFile(partitionItemFile.getParentFile()))
        return fileName; 
      if (isTftpOccupied) {
        Display.getDefault().syncExec((Runnable)new Object(fileName));
        return "";
      } 
      String localFilePath = ToolUtils.getInstallPath() + ToolUtils.getInstallPath() + File.separator;
      File localFile = new File(localFilePath);
      if (FileUtils.copy(null, partitionItemFile, localFile))
        return localFilePath; 
      Display.getDefault().syncExec((Runnable)new Object(fileName));
      return "";
    } 
    return fileName;
  }
  
  private static boolean canWriteFile(File parentFile) {
    File file = null;
    try {
      file = File.createTempFile("temp", ".temp", parentFile);
    } catch (IOException e) {
      return false;
    } finally {
      if (file != null && file.exists())
        FileUtils.deleteFile(file); 
    } 
    return true;
  }
  
  public static List<PartitionItem> getYaffsFileXmlList(PartitionItem item, long[] yaffsPageAndOobSize, boolean isTftpOccupied) throws IOException {
    File file = item.getFile();
    String fileName = file.getCanonicalPath();
    byte[] sparseFileXmlList = new byte[30000];
    boolean isDeleteTempFile = false;
    String canSplitFilePath = getCanSplitFilePath(isTftpOccupied, fileName, file);
    if (canSplitFilePath.isEmpty())
      return new ArrayList<>(); 
    if (!fileName.equals(canSplitFilePath))
      isDeleteTempFile = true; 
    fileName = canSplitFilePath;
    try {
      int spiltFileSize = Integer.parseInt(String.valueOf(ToolUtils.getSplitFileSize()));
      CallLibBootromWorker.setPacketSize(spiltFileSize);
      if (!CallLibBootromWorker.splitYaffsFile(fileName, item.getFileSystem().name(), item.getStart(), item
          .getLength(), yaffsPageAndOobSize)) {
        Log.error(NLS.bind(Messages.fileSpiltFailed, fileName));
        return new ArrayList();
      } 
    } finally {
      CallLibBootromWorker.getSparseFileXmlList(fileName, sparseFileXmlList);
      if (isDeleteTempFile)
        FileUtils.deleteFile(new File(canSplitFilePath)); 
    } 
    return getPartitionItemList(fileName, sparseFileXmlList);
  }
  
  private static List<PartitionItem> getPartitionItemList(String fileName, byte[] sparseFileXmlList) {
    List<PartitionItem> partitionItemList = new ArrayList<>();
    String[] str = (new String(sparseFileXmlList, StandardCharsets.UTF_8)).split(",");
    int i;
    for (i = 0; i < str.length && 
      i + 1 != str.length; i += 2) {
      PartitionItem partitionItem = new PartitionItem();
      partitionItem.setLength(Long.parseLong(str[i]));
      String filePath = str[i + 1];
      File file = new File(filePath);
      if (!file.exists() && fileName.lastIndexOf(File.separator) != -1 && filePath
        .lastIndexOf(File.separator) != -1) {
        filePath = fileName.substring(0, fileName.lastIndexOf(File.separator)) + fileName.substring(0, fileName.lastIndexOf(File.separator));
        file = new File(filePath);
      } 
      partitionItem.setFile(file);
      partitionItemList.add(partitionItem);
    } 
    return partitionItemList;
  }
  
  public static boolean isAndroidSparseFile(File file) {
    if (file == null || !file.exists())
      return false; 
    if (file.length() < 28L)
      return false; 
    RandomAccessFile randFile = null;
    SparseHeader header = new SparseHeader();
    try {
      randFile = new RandomAccessFile(file, "r");
      byte[] data = new byte[28];
      int readLen = randFile.read(data);
      if (readLen <= 0)
        return false; 
      header.fromBytes(data);
    } catch (IOException e) {
      return false;
    } finally {
      try {
        if (randFile != null)
          randFile.close(); 
      } catch (IOException iOException) {}
    } 
    if (header.magic == -316211398 && header.major_version == 1 && header.file_hdr_sz == 28)
      return true; 
    return false;
  }
  
  public static boolean isSparse(FileSystem fileSystem) {
    return (fileSystem == FileSystem.TAT32 || fileSystem == FileSystem.ext3 || fileSystem == FileSystem.f2fs || fileSystem == FileSystem.erofs);
  }
  
  public static boolean isAndroidSparseFile(byte[] data) {
    if (data.length < 28)
      return false; 
    SparseHeader header = new SparseHeader();
    header.fromBytes(data);
    if (header.magic == -316211398 && header.major_version == 1 && header.file_hdr_sz == 28)
      return true; 
    return false;
  }
  
  public static File unsparse(File file) {
    String fillData = null;
    if (ToolUtils.isCmdLine()) {
      fillData = String.valueOf(PreferenceConfig.getImgFillData());
    } else {
      IPreferenceStore preferences = FastbootPreferences.getPreferenceStore();
      fillData = preferences.getString("vendor.burn.config.imageFill");
    } 
    String newFilePath = file.getParent() + file.getParent() + File.separator + ".unsparse";
    long byteNum = 0L;
    RandomAccessFile randFile = null;
    RandomAccessFile outFile = null;
    try {
      randFile = new RandomAccessFile(file, "r");
      outFile = new RandomAccessFile(newFilePath, "rw");
      byte[] sparseHeaderData = new byte[28];
      randFile.seek(0L);
      int readLen = randFile.read(sparseHeaderData);
      if (readLen != sparseHeaderData.length)
        return null; 
      outFile.seek(0L);
      SparseHeader sparseHeader = new SparseHeader();
      sparseHeader.fromBytes(sparseHeaderData);
      byteNum += 28L;
      for (int i = 0; i < sparseHeader.total_chunks; i++) {
        byte[] chunkHeaderData = new byte[12];
        int read = randFile.read(chunkHeaderData);
        if (read < 0)
          System.out.println("Read Error"); 
        SparseChunkHeader chunkHeader = new SparseChunkHeader();
        chunkHeader.fromBytes(chunkHeaderData);
        byteNum += 12L;
        System.out.println("CHUNK: " + i + " type = " + chunkHeader.chunk_type + " chunk_sz = " + chunkHeader.chunk_sz + " Total Bytes = " + byteNum);
        if (chunkHeader.chunk_type == -13631) {
          int chunkLen = chunkHeader.chunk_sz * sparseHeader.blk_sz;
          int chunkDataSpaceCount = (chunkLen + 5242880 - 1) / 5242880;
          int chunkDataSpaceSpace = chunkLen % 5242880;
          if (chunkDataSpaceCount == 0) {
            byte[] chunkData = new byte[chunkLen];
            read = randFile.read(chunkData);
            if (read < 0)
              System.out.println("Read Error"); 
            outFile.write(chunkData);
            byteNum += chunkLen;
          } else {
            for (int j = 0; j < chunkDataSpaceCount; j++) {
              int len;
              byte[] chunkData = null;
              if (j == chunkDataSpaceCount - 1) {
                if (chunkDataSpaceSpace == 0) {
                  chunkData = new byte[5242880];
                  len = 5242880;
                } else {
                  chunkData = new byte[chunkDataSpaceSpace];
                  len = chunkDataSpaceSpace;
                } 
              } else {
                chunkData = new byte[5242880];
                len = 5242880;
              } 
              read = randFile.read(chunkData);
              if (read < 0)
                System.out.println("Read Error"); 
              outFile.write(chunkData);
              byteNum += len;
            } 
          } 
        } else if (chunkHeader.chunk_type == -13629) {
          int chunkLen = chunkHeader.chunk_sz * sparseHeader.blk_sz;
          for (int n = 0; n < chunkHeader.chunk_sz; n++) {
            byte[] bytes = new byte[sparseHeader.blk_sz];
            if (fillData.equals("0xFF"))
              bytes = BytesUtils.genBytesWithFF(sparseHeader.blk_sz); 
            outFile.write(bytes);
          } 
          byteNum += chunkLen;
        } else if (chunkHeader.chunk_type == -13630) {
          byte[] fill_val = new byte[4];
          int len = randFile.read(fill_val, 0, fill_val.length);
          if (len < 0)
            System.out.println("Read Error."); 
          byte[] fill_data = BytesUtils.genBytesFillValues(chunkHeader.chunk_sz * sparseHeader.blk_sz, fill_val);
          outFile.write(fill_data);
          byteNum += fill_val.length;
        } else {
          outFile.close();
          FileUtils.deleteFile(new File(newFilePath));
          return null;
        } 
      } 
    } catch (IOException e) {
      return null;
    } finally {
      if (randFile != null)
        try {
          randFile.close();
        } catch (IOException iOException) {} 
      if (outFile != null)
        try {
          outFile.close();
        } catch (IOException iOException) {} 
    } 
    return new File(newFilePath);
  }
}
